#ifndef _OSLIBE_H
#define _OSLIBE_H

#include <oslib/oslib.h>

#ifdef __cplusplus
extern "C" {
#endif

//////////////////////////////////////////////////////////////////////////
//								LES CONSTANTES							//
//////////////////////////////////////////////////////////////////////////

// Les angles

#ifndef M_PI
	#define M_PI	3.14159265358979323846f
	#define M_PI_2	1.57079632679489661923f
	#define M_PI_4	0.785398163397448309616f
	#define M_1_PI	0.318309886183790671538f
	#define M_2_PI	0.636619772367581343076f
#endif

// Les blending mode

#define OSLE_BLENDMODE_ADDITIVE OSL_FX_ADD|OSL_FX_COLOR
#define OSLE_BLENDMODE_NORMAL   OSL_FX_ALPHA|OSL_FX_COLOR

// Les Couleurs

#define OSLE_STARTING_COLOR 0
#define OSLE_ENDING_COLOR	1

//////////////////////////////////////////////////////////////////////////
//								LES STRUCTURES							//
//////////////////////////////////////////////////////////////////////////

// LES COULEURS

/// OSLE_PRECISE_COLOR holds the componants of a color
typedef struct 
{
	float r;		///< The red componant of the color
	float g;		///< The green componant of the color
	float b;		///< The blue componant of the color
	float a;		///< The alpha value of the color
} OSLE_PRECISE_COLOR;

/// OSLE_VECTOR holds the data for a vector
typedef struct OSLE_VECTOR
{
	float x;		///< The horizontal weight of the vector
	float y;		///< The vertical weight of the vector
} OSLE_VECTOR;

// LES PARTICULES

/// OSLE_PARTICLE holds the data for a particle
typedef struct OSLE_PARTICLE
{
	OSLE_VECTOR	vecLocation;		///< The position of the particle
	OSLE_VECTOR	vecVelocity;		///< The velocity of the particle

	float		gravity;
	float		radialAccel;
	float		tangentialAccel;

	float		spin;
	float		spinDelta;

	float		size;
	float		sizeDelta;

	float		age;
	float		terminalAge;

	OSLE_PRECISE_COLOR	color;
	OSLE_PRECISE_COLOR	colorDelta;

	struct OSLE_PARTICLE*	prev;		///< The previous particle
	struct OSLE_PARTICLE*	next;		///< The next particle

} OSLE_PARTICLE;

/// OSLE_EMTTER holds the data for a particles emitter
typedef struct OSLE_EMITTER
{
	struct OSLE_PARTICLE*	particles;			///< The first particle of the system
	/*struct*/ OSL_IMAGE*		particlesImage;		///< The image of a particle 
	struct OSLE_VECTOR      vecLocation;		///< The position of the emitter
	struct OSLE_VECTOR      vecPrevLocation;	///< The previous location of the emitter
	short		particlesCount;					///< The number of alive particles
	int			emission;						///< The number of emitted particle in a second
	int			blendMode;						///< The blend mode of the particles
	float		particleLifeMin;				///< The minimum period of lifetime for a specific particle in seconds. 
	float		particleLifeMax;				///< The maximum period of lifetime for a specific particle in seconds. 
	float		direction;						///< This is an angle in radians, defining the particles emitting direction.
	float		spread;							///< This is an angle in radians, defining the sector within which the particles are emitted.					
	float		speedMin;						///< The minimum starting speed for a specific particle in pixels per second. 
	float		speedMax;						///< The maximum starting speed for a specific particle in pixels per second. 
	float		gravityMin;						///< The minimum gravity value for a specific particle. The gravity is basically just an acceleration along the y-axis.
	float		gravityMax;						///< The maximum gravity value for a specific particle. The gravity is basically just an acceleration along the y-axis.
	float		radialAccelMin;					///< The minimum value of the acceleration directed towards the particle system's center for a specific particle. 
	float		radialAccelMax;					///< The maximum value of the acceleration directed towards the particle system's center for a specific particle. 
	float		tangentialAccelMin;				///< The minimum value of the acceleration directed perpendicularly to the radial acceleration for a specific particle. 
	float		tangentialAccelMax;				///< The minimum value of the acceleration directed perpendicularly to the radial acceleration for a specific particle. 
	float		sizeStart;						///< The starting size of a particle
	float		sizeEnd;						///< The ending size of a particle
	float		sizeVariation;					///< The variation of the ending and starting size
	float		colorVar;						///< The variation of the ending and starting color
	float		alphaVar;						///< The variation on the alpha Channel
	char*		particlesImageFileName;			///< The filename of the image of a particle
	OSLE_PRECISE_COLOR	colorStart;				///< The starting color of a particle
	OSLE_PRECISE_COLOR	colorEnd;				///< The ending color of a particle
	
	float		emissionResidue;				///< Reserved. The amout of residual computation for the emission

/*
	float		spinStart;
	float		spinEnd;
	float		spinVar;
*/

} OSLE_EMITTER;

// Les PIXEL FONTS

/// OSLE_PIXELCHAR holds the informations of a character in the bitmap file
typedef struct OSLE_PIXELCHAR
{
	char  c;			///< The character himself
	short x;		///< The horizontal position of the character in the bitmap file
	short y;		///< The vertical position of the character in the bitmap file
	short width;	///< The true width of the character in the bitmap file
	short height;	///< The true height of the character in the bitmap file
} OSLE_PIXELCHAR;

/// OSLE_PIXELFONT holds the informations of the font
typedef struct OSLE_PIXELFONT
{
	char*			fontImageName;		///< The image name of the the image that holds the bitmap of the font
	OSL_IMAGE*		fontImage;			///< The oslib image containing the bitmap of the font
	short			fontWidth;			///< The width of a character in the bitmap
	short			fontHeight;			///< The height of a character in the bitmap
	char*			charMap;			///< The different characters in the bitmap
	short			charSpacing;			///< The space beetween two chars when drawing text
	OSLE_PIXELCHAR	charTab[256];		///< Array that holds the information of each characters
} OSLE_PIXELFONT;

// LES FICHIERS INI
/// OSLE_INI_ENTRY holds the informations of a line of an ini file (a NamedValue)
typedef struct OSLE_INI_ENTRY
{
	char* name;			///< Parameter's name
	char* value;		///< Value of the parameter
} OSLE_INI_ENTRY;

/// OSLE_INI_SECTION holds a section of a ini file
typedef struct OSLE_INI_SECTION
{
	char* name;					///< The section's name
	short valuesCount;			///< The number of values in the section
	OSLE_INI_ENTRY** values;	///< The different values in the section
} OSLE_INI_SECTION;


typedef struct OSLE_INI_FILE
{
	char* fileName;					///< The fileName of the configuration file
	short sectionsCount;			///< The number of section in the ini file
	OSLE_INI_SECTION** sections;	///< The collections of sections
} OSLE_INI_FILE;

//////////////////////////////////////////////////////////////////////////
//								LES FONCTIONS							//
//////////////////////////////////////////////////////////////////////////

/**
* \defgroup emitter Particles Emitter
*/
/*@{*/

/// <summary>Create a particles emitter</summary>
/// <param name="ImageFileName">The image file that hold the representation of a particle.</param>
/// <remarks>
/// Create in memory the emitter, and load in VRAM the image for the particle.<br>
/// If the image filename is NULL, then the emitter assume that you will affect by yourself the field particleImage.
/// </remarks>
OSLE_EMITTER* osleCreateEmitter(char* ImageFileName);

/// <summary>Delete the emitter from the memory</summary>
/// <param name="emitter">The emitter to delete.</param>
/// <remarks>If you affected by yourself the particlesImage member, the image is not deleted from memory. In this case you MUST delete it by yourself.</remarks>
void osleDeleteEmitter(OSLE_EMITTER* emitter);

/// <summary>Update the particles with the emitter settings, and draw the particles</summary>
/// <param name="emitter">The emitter to update</param>
/// <param name="deltaTime">The time of a frame in your application (generally 1.0f/60.0f)</param>
void osleUpdateParticles(OSLE_EMITTER* emitter, float deltaTime);

/// <summary>Update the particles with the emitter settings, and draw the particles</summary>
/// <param name="emitter">The emitter to update</param>
/// <param name="color">The color to update (starting or ending color</param>
/// <param name="r">The red component of the new color</param>
/// <param name="g">The green component of the new color</param>
/// <param name="b">The blue component of the new color</param>
/// <param name="a">The alpha value of the new color</param>
void osleSetEmitterColor(OSLE_EMITTER* emitter, short color, short r, short g, short b, short a);

/*@}*/

/**
* \defgroup pixelfont Pixel Fonts
*/
/*@{*/
/// <summary>Create a pixel font</summary>
/// <param name="fileName">The image file that hold the characters.</param>
/// <param name="fontWidth">The width of a character in the image.</param>
/// <param name="fontHeight">The height of a character in the image.</param>
/// <param name="charMap">The list of the different characters in the image</param>
/// <remarks>
/// Create in memory the pixel font, and load in VRAM the image for the characters.<br>
/// If the image filename is NULL, then the pixelFont assume that you will affect by yourself the field fontImage.<br>
/// When you affect by yourself the fontImage, then you must call the osleInitPixelFont fonction.
/// </remarks>
OSLE_PIXELFONT* osleCreatePixelFont(char* fileName, short fontWidth, short fontHeight, char* charMap);

/// <summary>Initialize a pixel font</summary>
/// <param name="pf">The pixelFont to initialise</param>
/// <param name="fontWidth">The width of a character in the image.</param>
/// <param name="fontHeight">The height of a character in the image.</param>
/// <param name="charMap">The list of the different characters in the image</param>
/// <remarks>
/// This fonction compute and affect the different characters (charTab member) of the pixel font.
/// </remarks>
void osleInitPixelFont(OSLE_PIXELFONT* pf, short fontWidth, short fontHeight, char* charMap);

/// <summary>Delete the pixelFont from the memory</summary>
/// <param name="pf">The pixelFont to delete</param>
/// <remarks>If you affected by yourself the fontImage member, the image is not deleted from memory. In this case you MUST delete it by yourself.</remarks>
void osleDeletePixelFont(OSLE_PIXELFONT* pf);

/// <summary>Draw the string to the screen, with the specidied pixel font</summary>
/// <param name="pf">The pixelFont to use</param>
/// <param name="x">The horizontal position where the string has to be drawn</param>
/// <param name="y">The vertical position where the string has to be drawn</param>
/// <param name="string">The string to draw</param>
void oslePixelFontDrawSimple(OSLE_PIXELFONT* pf, short x, short y, char* string);

/// <summary>Get the width in pixels of a text using the given pixelFont</summary>
/// <param name="pf">The pixelFont to use</param>
/// <param name="string">The string to draw</param>
short oslePixelFontGetWidth(OSLE_PIXELFONT* pf, char* string);

/*@}*/

/**
* \defgroup inifile Configuration Files
*/
/*@{*/

/// <summary>Load a configuration file (.INI), and make a copy in memory to get and set values in this file</summary>
/// <param name="fileName">The file name of the configuration file</param>
OSLE_INI_FILE* osleLoadIniFile(char* fileName);

/// <summary>Create an emty configuration file</summary>
/// <param name="fileName">The file name of the configuration file</param>
OSLE_INI_FILE* osleCreateIniFile(char* fileName);

/// <summary>Delete from memory a configuration file</summary>
/// <param name="iniFile">The configuration file to delete</param>
void osleDeleteIniFile(OSLE_INI_FILE* iniFile);

/// <summary>Save to disk the configuration file</summary>
/// <param name="iniFile">The configuration file to save</param>
void osleIniFileSaveToDisk(OSLE_INI_FILE* iniFile);

/// <summary>Update or add a new integer value in a configuration file</summary>
/// <param name="iniFile">The configuration file to update</param>
/// <param name="section">The name of section where to add or update the value</param>
/// <param name="name">The name of the value to add or update</param>
/// <param name="value">The new value</param>
void osleIniFileSetInt(OSLE_INI_FILE* iniFile, char *section, char *name, int value);

/// <summary>Retreive a integer value in a configuration file</summary>
/// <param name="iniFile">The configuration file to update</param>
/// <param name="section">The name of section to search</param>
/// <param name="name">The name of the value to retreive</param>
/// <param name="defaultValue">The value returned if the requested value is not found</param>
int osleIniFileGetInt(OSLE_INI_FILE* iniFile, char *section, char *name, int defaultValue);

/// <summary>Update or add a new float value in a configuration file</summary>
/// <param name="iniFile">The configuration file to update</param>
/// <param name="section">The name of section where to add or update the value</param>
/// <param name="name">The name of the value to add or update</param>
/// <param name="value">The new value</param>
void osleIniFileSetFloat(OSLE_INI_FILE* iniFile, char *section, char *name, float value);

/// <summary>Retreive a float value in a configuration file</summary>
/// <param name="iniFile">The configuration file to update</param>
/// <param name="section">The name of section to search</param>
/// <param name="name">The name of the value to retreive</param>
/// <param name="defaultValue">The value returned if the requested value is not found</param>
float osleIniFileGetFloat(OSLE_INI_FILE* iniFile, char *section, char *name, float defaultValue);

/// <summary>Update or add a new string value in a configuration file</summary>
/// <param name="iniFile">The configuration file to update</param>
/// <param name="section">The name of section where to add or update the value</param>
/// <param name="name">The name of the value to add or update</param>
/// <param name="value">The new value</param>
void osleIniFileSetString(OSLE_INI_FILE* iniFile, char *section, char *name, char* value);

/// <summary>Retreive a string value in a configuration file</summary>
/// <param name="iniFile">The configuration file to update</param>
/// <param name="section">The name of section to search</param>
/// <param name="name">The name of the value to retreive</param>
/// <param name="defaultValue">The value returned if the requested value is not found</param>
char* osleIniFileGetString(OSLE_INI_FILE* iniFile, char *section, char *name, char* defaultValue);

/*@}*/

#ifdef __cplusplus
}
#endif

#endif
